import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Tooltip, Modal, notification, Switch } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import moment from 'moment'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import zhCN from '@/locales/zh-CN/model.js'
import enUS from '@/locales/en-US/model.js'
import { getSearchForm } from '@/templates/zshare/formconfig'
import asyncComponent from '@/utils/asyncComponent'
import MKEmitter from '@/utils/events.js'
import SearchForm from './searchform'
import DragElement from './dragsearch'
import './index.scss'

const { confirm } = Modal

const FieldsComponent = asyncComponent(() => import('@/templates/sharecomponent/fieldscomponent'))

class SearchComponent extends Component {
  static propTpyes = {
    config: PropTypes.object,        // 配置信息
    updatesearch: PropTypes.func     // 更新
  }

  state = {
    dict: sessionStorage.getItem('lang') !== 'en-US' ? zhCN : enUS,
    searchlist: null,    // 搜索条件集
    sqlVerifing: false,  // sql验证中
    visible: false,      // 模态框控制
    showField: false,
    card: null           // 编辑中元素
  }

  /**
   * @description 搜索条件初始化
   */
  UNSAFE_componentWillMount () {
    this.setState({
      searchlist: fromJS(this.props.config.search).toJS()
    })
  }

  // /**
  //  * @description 监听到搜索条件复制时，触发搜索条件编辑
  //  */
  // UNSAFE_componentWillReceiveProps (nextProps) {
  //   const { searchlist } = this.state

  //   if (!is(fromJS(nextProps.config.search), fromJS(this.props.config.search)) && !is(fromJS(nextProps.config.search), fromJS(searchlist))) {
  //     let len = nextProps.config.search.length
  //     let item = nextProps.config.search[len - 1]
  //     if (item && item.focus) {
  //       this.handleSearch(item)
  //     }
  //     this.setState({searchlist: fromJS(nextProps.config.search).toJS()})
  //   }
  // }

  componentDidMount () {
    MKEmitter.addListener('plusSearch', this.plusSearch)
    MKEmitter.addListener('revert', this.revert)
  }

  revert = () => {
    this.setState({searchlist: fromJS(this.props.config.search).toJS()})
  }

  plusSearch = (MenuId, item, type) => {
    const { config } = this.props
    const { searchlist } = this.state

    if (MenuId !== config.uuid) return

    if (type === 'simple') {
      this.setState({
        searchlist: [...searchlist, item],
      }, () => {
        this.handleSearch(item)
      })
    } else if (type === 'multil') {
      let list = [...searchlist, ...item]
      list = list.filter(item => !item.origin) // 去除系统项
      
      this.setState({
        searchlist: list
      }, () => {
        this.props.updatesearch({...config, search: list})
      })
    }
  }

  /**
   * @description 搜索条件顺序调整，或拖拽添加
   */
  handleList = (list, card) => {
    const { config } = this.props

    if (card) {
      this.setState({searchlist: list})
      this.handleSearch(card)
    } else {
      this.setState({searchlist: list}, ()=> {  
        this.props.updatesearch({...config, search: list})
      })
    }
  }

  /**
   * @description 搜索条件编辑，获取搜索条件表单信息
   */
  handleSearch = (card) => {
    const { searchlist } = this.state
    let linkableFields = []

    searchlist.forEach(item => {
      if (item.uuid === card.uuid) return
      if (!['select', 'link', 'checkcard'].includes(item.type)) return
      if (item.type === 'checkcard' && item.multiple === 'true') return
      
      linkableFields.push({
        value: item.field,
        text: item.label
      })
    })

    this.setState({
      visible: true,
      card: card,
      formlist: getSearchForm(card, linkableFields)
    })
  }

  /**
   * @description 取消保存，如果元素为新添元素，则从序列中删除
   */
  editModalCancel = () => {
    const { config } = this.props
    const { card, searchlist } = this.state

    if (card.focus) {
      let _searchlist = searchlist.filter(item => item.uuid !== card.uuid)

      this.setState({
        card: null,
        searchlist: _searchlist,
        visible: false
      })
      this.props.updatesearch({...config, search: _searchlist})
    } else {
      this.setState({
        card: null,
        visible: false
      })
    }
  }

  /**
   * @description 搜索修改后提交保存
   * 1、去除系统默认搜索条件
   * 2、字段及提示文字重复校验
   * 3、更新下拉菜单可选集合
   * 4、下拉菜单数据源语法验证
   */
  handleSubmit = () => {
    const { config } = this.props
    let _searchlist = fromJS(this.state.searchlist).toJS()

    this.searchFormRef.handleConfirm().then(res => {
      let fieldrepet = false // 字段重复
      let labelrepet = false // 提示文字重复

      _searchlist = _searchlist.filter(item => !item.origin || item.uuid === res.uuid) // 去除系统项

      _searchlist = _searchlist.map(item => { // 数据更新及重复检测
        if (item.uuid !== res.uuid && res.field && item.field) {
          let itemFields = []
          if (item.type === 'text') {
            itemFields = item.field.split(',')
          } else if (item.type === 'group') {
            itemFields = [item.field, item.datefield]
          } else {
            itemFields = [item.field]
          }

          let resFields = []
          if (res.type === 'text') {
            resFields = res.field.split(',')
          } else if (res.type === 'group') {
            resFields = [res.field, res.datefield]
          } else {
            resFields = [res.field]
          }

          let setFields = Array.from(new Set([...itemFields, ...resFields]))

          if (setFields.length < itemFields.length + resFields.length && (res.type !== 'date' || item.type !== 'date')) {
            fieldrepet = true
          } else if (item.label === res.label) {
            labelrepet = true
          }
        }

        if (item.uuid === res.uuid) {
          return res
        } else {
          return item
        }
      })

      if (fieldrepet) {
        notification.warning({
          top: 92,
          message: '字段已存在!',
          duration: 5
        })
        return
      } else if (labelrepet) {
        notification.warning({
          top: 92,
          message: '名称已存在!',
          duration: 5
        })
        return
      }

      if (['checkcard', 'select', 'multiselect', 'link'].includes(res.type) && res.resourceType === '1' && /\s/.test(res.dataSource)) {
        this.setState({
          sqlVerifing: true
        })

        let param = {
          func: 's_debug_sql',
          exec_type: 'y',
          LText: res.dataSource
        }

        param.LText = param.LText.replace(/@\$|\$@/ig, '')
        
        param.LText = Utils.formatOptions(param.LText)
        param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
        param.secretkey = Utils.encrypt('', param.timestamp)

        if (window.GLOB.mainSystemApi && res.database === 'sso') {
          param.rduri = window.GLOB.mainSystemApi
        }
        
        Api.getLocalConfig(param).then(result => {
          if (result.status) {
            this.setState({
              sqlVerifing: false,
              searchlist: _searchlist,
              visible: false
            }, ()=> {
              this.props.updatesearch({...config, search: _searchlist})
            })
          } else {
            this.setState({sqlVerifing: false})
            
            Modal.error({
              title: result.message
            })
          }
        })
      } else {
        this.setState({
          searchlist: _searchlist,
          visible: false
        }, ()=> { 
          this.props.updatesearch({...config, search: _searchlist})
        })
      }
    })
  }

  /**
   * @description 搜索条件删除
   */
  deleteElement = (card) => {
    const { config } = this.props
    const { dict } = this.state
    let _this = this

    confirm({
      content: dict['model.confirm'] + dict['model.delete'] + ` - ${card.label} ？`,
      onOk() {
        let _searchlist = fromJS(_this.state.searchlist).toJS()

        _searchlist = _searchlist.filter(item => item.uuid !== card.uuid)

        _this.setState({
          searchlist: _searchlist
        }, () => {
          _this.props.updatesearch({...config, search: _searchlist})
        })
      },
      onCancel() {}
    })
  }

  onFieldChange = () => {
    const { showField } = this.state

    this.setState({
      showField: !showField
    })
  }


  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('revert', this.revert)
    MKEmitter.removeListener('plusSearch', this.plusSearch)
  }

  shouldComponentUpdate (nextProps, nextState) {
    if (!is(fromJS(this.state), fromJS(nextState))) {
      return true
    } else if (this.props.config.wrap) {
      return this.props.config.wrap.show !== nextProps.config.wrap.show
    } else {
      return this.props.config.setting.show !== nextProps.config.setting.show
    }
  }

  render() {
    const { config } = this.props
    const { dict, searchlist, visible, sqlVerifing, card, showField } = this.state

    let show = config.setting.show
    if (config.wrap) {
      show = config.wrap.show
    }

    return (
      <div className={'model-table-search-list length' + searchlist.length}>
        <Tooltip placement="bottomLeft" overlayClassName="middle" title="在左侧工具栏《搜索》中，选择对应搜索框拖至此处添加；或点击按钮《添加搜索条件》批量添加，选择批量添加时，需提前选择使用表。">
          <QuestionCircleOutlined style={{color: '#c49f47', position: 'relative', left: '-15px', top: '5px'}} />
        </Tooltip>
        <FieldsComponent config={{uuid: config.uuid, search: searchlist}} type="search" />
        <Switch checkedChildren={dict['model.switch.open']} unCheckedChildren={dict['model.switch.close']} defaultChecked={showField} onChange={this.onFieldChange} />
        <DragElement
          list={searchlist}
          show={show}
          showField={showField}
          handleList={this.handleList}
          handleMenu={this.handleSearch}
          deleteMenu={this.deleteElement}
        />
        {/* 编辑搜索条件 */}
        <Modal
          title={card && card.copyType === 'search' ?  '搜索条件-复制' : '搜索条件-编辑'}
          visible={visible}
          width={850}
          maskClosable={false}
          onOk={this.handleSubmit}
          confirmLoading={sqlVerifing}
          onCancel={this.editModalCancel}
          destroyOnClose
        >
          <SearchForm
            dict={dict}
            card={this.state.card}
            formlist={this.state.formlist}
            inputSubmit={this.handleSubmit}
            wrappedComponentRef={(inst) => this.searchFormRef = inst}
          />
        </Modal>
      </div>
    )
  }
}

export default SearchComponent